/*
 * =================================================================
 * Filename:             hidemodule.c
 * Description:          Hide modules loaded from /module for non opers
 * Created by:		 Harakiri <harakiri@overstack.fr>
 * Created from the 'hideserver' code source, by AngryWolf
 * =================================================================
 */

#include "config.h"
#include "struct.h"
#include "common.h"
#include "sys.h"
#include "numeric.h"
#include "msg.h"
#include "channel.h"
#include <time.h>
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#ifdef _WIN32
#include <io.h>
#endif
#include <fcntl.h>
#include "h.h"
#ifdef STRIPBADWORDS
#include "badwords.h"
#endif
#ifdef _WIN32
#include "version.h"
#endif

extern void			sendto_one(aClient *to, char *pattern, ...);
extern char			*my_itoa(int i);

#define DelOverride(cmd, ovr)	if (ovr && CommandExists(cmd)) CmdoverrideDel(ovr); ovr = NULL
#define DelHook(x)      	if (x) HookDel(x); x = NULL
#define ircstrdup(x,y)		if (x) MyFree(x); if (!y) x = NULL; else x = strdup(y)
#define ircfree(x)		if (x) MyFree(x); x = NULL

Cmdoverride			*AddOverride(char *msg, iFP cb);
static int			override_module(Cmdoverride *, aClient *, aClient *, int, char *[]);
static int			cb_test(ConfigFile *, ConfigEntry *, int, int *);
static int			cb_conf(ConfigFile *, ConfigEntry *, int);
static int			cb_rehash();
/* static int			cb_stats(aClient *sptr, char *stats); */

Hook				*HookConfTest, *HookConfRun;
Hook				*HookConfRehash;
Cmdoverride			*OvrModule;
ConfigItem_ulines		*HiddenServers;
extern MODVAR Module			*Modules;
static struct
{
	unsigned	disable_module : 1;
	char		*module_deny_message;
} Settings;

#ifndef STATIC_LINKING
static ModuleInfo	*MyModInfo;
 #define MyMod		MyModInfo->handle
 #define SAVE_MODINFO	MyModInfo = modinfo;
#else
 #define MyMod		NULL
 #define SAVE_MODINFO
#endif

ModuleHeader MOD_HEADER(hidemodule)
  = {
	"hidemodule",
	"hidemodule.c, v1 2010/11/01 Harakiri",
	"Hide modules from /module (for non-opers)",
	"3.2-b8-1",
	NULL 
    };

static void InitConf()
{
	memset(&Settings, 0, sizeof Settings);
}

static void FreeConf()
{
	ConfigItem_ulines	*h;
	ListStruct		*next;

	ircfree(Settings.module_deny_message);

	for (h = HiddenServers; h; h = (ConfigItem_ulines *) next)
	{
		next = (ListStruct *) h->next;
		DelListItem(h, HiddenServers);
		MyFree(h->servername);
		MyFree(h);
	}
}

DLLFUNC int MOD_TEST(hideserver)(ModuleInfo *modinfo)
{
	SAVE_MODINFO
	HookConfTest	= HookAddEx(modinfo->handle, HOOKTYPE_CONFIGTEST, cb_test);

	return MOD_SUCCESS;
}

DLLFUNC int MOD_INIT(hideserver)(ModuleInfo *modinfo)
{
	SAVE_MODINFO
	HiddenServers = NULL;
	InitConf();

	HookConfRun	= HookAddEx(modinfo->handle, HOOKTYPE_CONFIGRUN, cb_conf);
	HookConfRehash	= HookAddEx(modinfo->handle, HOOKTYPE_REHASH, cb_rehash);

        return MOD_SUCCESS;
}

DLLFUNC int MOD_LOAD(hideserver)(int module_load)
{
	int ret = MOD_SUCCESS;

	OvrModule	= AddOverride("module", override_module);

	if (!OvrModule)
		ret = MOD_FAILED;

	return ret;
}

DLLFUNC int MOD_UNLOAD(hideserver)(int module_unload)
{
	FreeConf();

	DelOverride("module", OvrModule);

	DelHook(HookConfRehash);
	DelHook(HookConfRun);
	DelHook(HookConfTest);

	return MOD_SUCCESS;
}

static int cb_rehash()
{
	FreeConf();
	InitConf();

	return 1;
}

static int cb_test(ConfigFile *cf, ConfigEntry *ce, int type, int *errs)
{
	ConfigEntry	*cep, *cepp;
	int		errors = 0;

	if (type == CONFIG_MAIN)
	{
		if (!strcmp(ce->ce_varname, "hidemodule"))
		{
			for (cep = ce->ce_entries; cep; cep = cep->ce_next)
			{
				if (!cep->ce_varname)
				{
					config_error("%s:%i: blank %s item",
						cep->ce_fileptr->cf_filename,
						cep->ce_varlinenum, ce->ce_varname);
					errors++;
					continue;
				}
				else if (!cep->ce_vardata)
				{
					config_error("%s:%i: %s::%s without value",
						cep->ce_fileptr->cf_filename,
						cep->ce_varlinenum,
						ce->ce_varname, cep->ce_varname);
					errors++;
					continue;
				}
				else if (!strcmp(cep->ce_varname, "disable-module"))
					;
				else if (!strcmp(cep->ce_varname, "deny-message"))
					;
				else
				{
					config_error("%s:%i: unknown directive hidemodule::%s",
						cep->ce_fileptr->cf_filename, cep->ce_varlinenum, cep->ce_varname);
					errors++;
				}
			}
			*errs = errors;
			return errors ? -1 : 1;
		}
	}

	return 0;
}

static int cb_conf(ConfigFile *cf, ConfigEntry *ce, int type)
{
	ConfigEntry		*cep, *cepp;
	ConfigItem_ulines	*ca;

	if (type == CONFIG_MAIN)
	{
		if (!strcmp(ce->ce_varname, "hidemodule"))
		{
			for (cep = ce->ce_entries; cep; cep = cep->ce_next)
			{
				if (!strcmp(cep->ce_varname, "disable-module"))
					Settings.disable_module = config_checkval(cep->ce_vardata, CFG_YESNO);
				else if (!strcmp(cep->ce_varname, "deny-message"))
				{
					ircstrdup(Settings.module_deny_message, cep->ce_vardata);
				}
				else
				  ircstrdup(Settings.module_deny_message, "*** /module command disabled for non-oper users");
		     	  }

			return 1;
		}
	}

	return 0;
}

Cmdoverride *AddOverride(char *msg, iFP cb)
{
	Cmdoverride *ovr = CmdoverrideAdd(MyMod, msg, cb);

#ifndef STATIC_LINKING
        if (ModuleGetError(MyMod) != MODERR_NOERROR || !ovr)
#else
        if (!ovr)
#endif
	{
#ifndef STATIC_LINKING
		config_error("Error replacing command %s when loading module %s: %s",
			msg, MOD_HEADER(hideserver).name, ModuleGetErrorStr(MyMod));
#else
		config_error("Error replacing command %s when loading module %s",
			msg, MOD_HEADER(hideserver).name);
#endif
		return NULL;
	}

	return ovr;
}


/* 
** /module override function
*/

static int override_module(Cmdoverride *ovr, aClient *cptr, aClient *sptr, int parc, char *parv[])
{
  aClient		*acptr;
  Module		*m = NULL;
  char			mod_msg[512];
  
  if (!IsClient(sptr))
    return CallCmdoverride(ovr, cptr, sptr, parc, parv);
  
  if (!IsAnOper(sptr) && Settings.disable_module)
    {
      if (Settings.module_deny_message)
	sendto_one(sptr, ":%s %s %s :%s",
		   me.name, IsWebTV(sptr) ? "PRIVMSG" : "NOTICE",
		   sptr->name, Settings.module_deny_message);
      return 0;
    }
  
  for (m = Modules; m; m = m->next)
    {
      snprintf(mod_msg, 512, "%s - %s %s", m->header->name, 
	       m->header->version, m->header->description);
      sendto_one(sptr, mod_msg);
    }
  return 0;
}
 
